@@ -1,11 +1,17 @@
 from easycv.io.output import save, show, show_grid
-from easycv.io.input import open_image, valid_image_source, get_image_array
+from easycv.io.input import (
+    open_image,
+    valid_image_source,
+    get_image_array,
+    random_dog_image,
+)
 
 __all__ = [
+    "get_image_array",
+    "open_image",
+    "random_dog_image",
     "save",
     "show",
     "show_grid",
-    "open_image",
     "valid_image_source",
-    "get_image_array",
 ]
